package controler;

import model.Arena;
import view.Renderer;


public class GameControler implements Runnable{
	private static GameControler instance = null;
	private boolean running;
	private int sleepTime;
	
	/**
	 * Funo inicial do programa.
	 * Inicializa o renderizador e cria uma thread para rodar o programa.
	 * @param args Argumentos de entrada. Nesse caso nada  recebido.
	 */
	public static void main(String[] args) {
		new Renderer();
		
		(new Thread(GameControler.instanceOf())).run();
	}
	
	/**
	 * Construtor da classe GameControler.
	 * Inicializa as variveis running e sleepTime.
	 * Armazena em instance a instncia da classe GameControler, para que exista apenas uma referncia.
	 */
	public GameControler(){
		GameControler.instance = this;
		this.running = true;
		this.sleepTime = 20;
	}
	
	/**
	 * Mtodo de acesso  instancia da classe GameControler.
	 * Se a instncia for nula, ela  inicializada.
	 * @return Uma instncia da classe GameControler.
	 */
	public static GameControler instanceOf(){
		if (instance == null)
			new GameControler();
		return instance;
	}
	
	/**
	 * Altera o valor de running para falso, terminando assim o ciclo de execuo do programa.
	 */
	public void endProgram(){
		this.running = false;
	}
	
	/**
	 * Diminui o tempo que o programa dorme entre iteraes. Mnimo de 0 milisegundos.
	 */
	public void faster(){
		if (this.sleepTime > 0){
			this.sleepTime -= 2;
		}
	}
	
	/**
	 * Aumenta o tempo que o programa dorme entre iteraes. Mximo de 200 milisegundos.
	 */
	public void slower(){
		if (this.sleepTime < 200){
			this.sleepTime += 20;
		}
	}
	
	/**
	 * Mtodo responsvel por manter o jogo rodando.
	 * Enquanto o jogo estiver correndo ele executa um passo de ao de cada
	 * personagem em jogo e dorme um tempo estipulado em sleepTime.
	 */
	public void run() {
		while (this.running){
			for (int i = 0; i < Arena.instanceOf().getBobs().size(); i++){
				Arena.instanceOf().getBobs().get(i).performAction();
			}
			try {
				Thread.sleep(this.sleepTime);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
}
