package controler;

import model.Arena;
import model.Player;
import model.Actions.*;

public class InputHandler {
	private static InputHandler instance = null;
	
	/**
	 * Construtor da classe InputHandler.
	 * Armazena em instance a instncia da classe InputHandler, para que exista apenas uma referncia.
	 */
	public InputHandler(){
		InputHandler.instance = this;
	}
	
	/**
	 * Mtodo de acesso  instancia da classe InputHandler.
	 * Se a instncia for nula, ela  inicializada.
	 * @return Uma instncia da classe InputHandler.
	 */
	public static InputHandler instanceOf(){
		if (instance == null)
			new InputHandler();
		return instance;
	}
	
	/**
	 * Mtodo para o tratamento do input do usurio.
	 * Quando o usurio apertar algum boto correspondente a uma ao esse mtodo
	 *  chamado para fazer o tratamento adequando, avisando o personagem ativo 
	 * qual  a prxima ao a ser executada.
	 * @param name O nome da ao a ser executada.
	 */
	public void pressButton(String name){
		if (Arena.instanceOf().getSelected() != null){
			if (name == "Jump"){
				Arena.instanceOf().getSelected().setNextAction(new Jump("Jump", Arena.instanceOf().getSelected(), null));
			}
			else if (name == "Clone"){
				if (!Arena.instanceOf().getSelected().isHasCloned()){
					Arena.instanceOf().getSelected().setNextAction(new Clone("Clone", Arena.instanceOf().getSelected(), null));
				}
			}
			else if (name == "Punch"){
				Arena.instanceOf().getSelected().setNextAction(new Punch("Punch", Arena.instanceOf().getSelected(), null));
			}
			else if (name == "Kick"){
				if (Arena.instanceOf().getSelected().isHasCloned()){
					for (int i = 0; i < Arena.instanceOf().getBobs().size(); i++){
						Arena.instanceOf().getBobs().get(i).setNextAction(new Kick("Kick", Arena.instanceOf().getBobs().get(i), null));
					}
				}
				else {
					Arena.instanceOf().getSelected().setNextAction(new Kick("Kick", Arena.instanceOf().getSelected(), null));
				}
			}
			else if (name == "Wave"){
				Arena.instanceOf().getSelected().setNextAction(new Wave("Wave", Arena.instanceOf().getSelected(), null));
			}
			else if (name == "Hadoken"){
				if (Arena.instanceOf().getSelected().isHasCloned()){
					for (int i = 0; i < Arena.instanceOf().getBobs().size(); i++){
						Arena.instanceOf().getBobs().get(i).setNextAction(new Hadoken("Hadoken", Arena.instanceOf().getBobs().get(i), null));
					}
				}
				else {
					Arena.instanceOf().getSelected().setNextAction(new Hadoken("Hadoken", Arena.instanceOf().getSelected(), null));
				}
			}
			else {
				System.out.println("View mandando besteira " + name);
			}
		}
	}
	
	/**
	 * Mtodo para mudar o personagem selecionado.
	 * @param player O novo personagem selecionado.
	 */
	public void selectPlayer(Player player){
		Arena.instanceOf().setSelected(player);
	}
	
	/**
	 * Mtodo para movimentao do personagem selecionado. 
	 * Causa o personagem a andar para as coordenadas recebidas.
	 * @param x A posio do eixo x no qual o usurio clicou.
	 * @param y A posio do eixo y no qual o usurio clicou.
	 */
	public void oneClickOn(double x, double y){
		Player target;
		target = new Player();
		target.setLocation(x, y);
		Arena.instanceOf().getSelected().setNextAction(new Move("Move", Arena.instanceOf().getSelected(), target));
	}
	
	/**
	 * Mtodo para movimentao do personagem selecionado. 
	 * Causa o personagem a correr para as coordenadas recebidas.
	 * @param x A posio do eixo x no qual o usurio clicou.
	 * @param y A posio do eixo y no qual o usurio clicou.
	 */
	public void twoClickOn(double x, double y){
		Player target;
		target = new Player();
		target.setLocation(x, y);
		Arena.instanceOf().getSelected().setNextAction(new Run("Run", Arena.instanceOf().getSelected(), target));
	}
	
	/**
	 * Mtodo para movimentao do personagem selecionado. 
	 * Causa o teleporte instantneo para as coordenadas recebidas.
	 * @param x A posio do eixo x no qual o usurio clicou.
	 * @param y A posio do eixo y no qual o usurio clicou.
	 */
	public void threeClickOn(double x, double y){
		Player target;
		target = new Player();
		target.setLocation(x, y);
		Arena.instanceOf().getSelected().setNextAction(new Teleport("Teleport", Arena.instanceOf().getSelected(), target));
	}
}
