package model.Actions;

import model.Arena;
import model.Player;



public class Move extends Action{
	private boolean ending = false;
	private double speed;
	
	/**
	 * Construtor da classe Move.
	 * @param name O nome da ao.
	 * @param active O personagem que vai executar a ao.
	 * @param target Um personagem falso apenas para armazenar a posio que se deseja ir. 
	 */
	public Move(String name, Player active, Player target) {
		super(name, active, target);
	}
	
	/**
	 * Funo que executa a ao apropriadamente.
	 * Enquanto o personagem no estiver perto do ponto desejado, ele anda a distncia
	 * determinada durante a criao dessa ao. Se ele estiver prximo ele anda
	 * cada vez menos at chegar no ponto desejado.
	 * Ao chegar na posio desejada a ao acaba.
	 */
	protected void execute() {
		if (this.active.getX() != this.target.getX() || this.active.getY() != this.target.getY()){
			this.active.setMoving(true);
			if (this.ending){
				if (this.step == 0){
					this.active.setMoving(false);
					this.active.endAction();
				}
				this.active.setLocation(
						this.active.getX() + (this.target.getX() - this.active.getX())/(40 - this.step), 
						this.active.getY() + (this.target.getY() - this.active.getY())/(40 - this.step));
			}
			else{
				this.active.setLocation(
						this.active.getX() + this.active.getDirX()*this.speed, 
						this.active.getY() + this.active.getDirY()*this.speed);
			}
		}
		else{
			this.active.endAction();
			this.active.setMoving(false);
		}
	}
	
	/**
	 * Inicializa a direo do personagem, sua velocidade e poe uma aura no ponto desejado.
	 * Caso ele esteja prximo do ponto desejado, ele atribui verdadeiro para uma varivel que indica a finalizao da ao.
	 */
	public void initialize(){
		double distance;
		distance = Math.sqrt((target.getX() - active.getX()) * (target.getX() - active.getX()) + (target.getY() - active.getY()) * (target.getY() - active.getY()));
		active.setDirection((target.getX() - active.getX())/distance, (target.getY() - active.getY())/distance);
		this.speed = 1.5/40.0;
		Arena.instanceOf().setCliked(true);
		Arena.instanceOf().setAuraPoint(this.target.getX(), this.target.getY());
		if (distance < 1.5){
			this.ending = true;
		}
	}
	
	/**
	 * Faz um passo de ao.
	 * Verifica se e quando essa ao deve terminar de acordo com a prxima ao
	 * a ser executada, que fica armazenado em flag.
	 */
	public void step(){
		double distance;
		this.step = (step + 1)%40;
		this.execute();
		if (this.step == 0){
			distance = Math.sqrt((target.getX() - active.getX()) * (target.getX() - active.getX()) + (target.getY() - active.getY()) * (target.getY() - active.getY()));
			if (distance < 1.5){
				this.ending = true;
			}
			if (!this.flag.equals("")){
				this.active.setMoving(false);
				this.active.endAction();
			}
		}
	}
	
	/**
	 * Determina qual o passo inicial da animao da ao.
	 * Depende de qual era a ao anterior para decidir isso.
	 */
	public void setStartStepFor(Action action) {
		this.step = 0;
	}
}
