package model.Actions;

import model.Arena;
import model.Player;

public class Run extends Action {
	private int cycle;
	private String state;
	private double speed;
	
	/**
	 * Construtor da classe Run.
	 * @param name O nome da ao.
	 * @param active O personagem que vai executar a ao.
	 * @param target Um personagem falso apenas para armazenar a posio que se deseja ir. 
	 */
	public Run(String name, Player active, Player target) {
		super(name, active, target);
	}
	
	/**
	 * Funo que executa a ao apropriadamente.
	 * Enquanto o personagem no estiver perto do ponto desejado, ele anda a distncia
	 * determinada durante a criao dessa ao. Se ele estiver prximo ele anda
	 * cada vez menos at chegar no ponto desejado.
	 * Ao chegar na posio desejada a ao acaba.
	 */
	protected void execute() {
		if (this.active.getX() != this.target.getX() || this.active.getY() != this.target.getY()){
			this.active.setMoving(true);
			if (this.state.equals("Ending")){
				if (this.step == 0){
					this.active.setMoving(false);
					this.active.endAction();
				}
				this.active.setLocation(
						this.active.getX() + (this.target.getX() - this.active.getX())/(10 - this.step), 
						this.active.getY() + (this.target.getY() - this.active.getY())/(10 - this.step));
			}
			else{
				if (this.state.equals("Starting")){
					if (this.step == 0){
						this.speed = 0.1125;
						this.state = "Running";
						this.cycle = 40;
					}
					else {
						this.speed = this.step * 0.01125;
					}
				}
				if (this.state.equals("Finish")){
					if (this.step == 0){
						this.active.endAction();
						this.active.setMoving(false);						
					}
					this.speed = (10 - this.step) * 0.01125;
				}
				this.active.setLocation(
						this.active.getX() + this.active.getDirX()*this.speed, 
						this.active.getY() + this.active.getDirY()*this.speed);
			}
		}
		else{
			this.active.endAction();
			this.active.setMoving(false);
		}
	}

	/**
	 * Inicializa a direo do personagem, coloca uma aura no ponto desejado.
	 * E calcula qual o ciclo e a velocidade de movimentao.
	 */
	public void initialize(){
		double distance;
		distance = Math.sqrt((target.getX() - active.getX()) * (target.getX() - active.getX()) + (target.getY() - active.getY()) * (target.getY() - active.getY()));
		active.setDirection((target.getX() - active.getX())/distance, (target.getY() - active.getY())/distance);
		Arena.instanceOf().setCliked(true);
		Arena.instanceOf().setAuraPoint(this.target.getX(), this.target.getY());
		if (this.state.equals("Running")){
			if (distance > 4.5){
				this.cycle = 40;
				this.speed = 0.1125;
			}
			else if(distance > 0.61875) {
				this.cycle = 40;
				this.speed = (distance - 0.6187)/40;
			}
			else {
				this.state = "Ending";
				this.cycle = 10;
				this.speed = 0;
			}
		}
	}
	
	/**
	 * Faz um passo de ao.
	 * Verifica se e quando essa ao deve terminar de acordo com a prxima ao
	 * a ser executada, que fica armazenado em flag.
	 */
	public void step(){
		double distance;
		this.step = (step + 1)%this.cycle;
		this.execute();
		if (this.step == 0){
			distance = Math.sqrt((target.getX() - active.getX()) * (target.getX() - active.getX()) + (target.getY() - active.getY()) * (target.getY() - active.getY()));
			if (this.flag.equals("Move")){
				this.active.endAction();
			}
			else if (this.flag.equals("RunningJump")){
				this.active.endAction();
			}
			else if (this.flag.equals("Teleport")){
				this.active.setMoving(false);
				this.active.endAction();
			}
			else if (this.flag.equals("Run")){
				this.active.endAction();
			}
			else if (!this.flag.equals("")){
				this.state = "Finish"; 
				this.cycle = 10;
			}
			this.speed = (distance - 0.6187)/40;
			if (this.speed > 0.1125){
				this.speed = 0.1125;
			}
			if (distance < 0.61875 || this.speed < 0.025) {
				this.state = "Ending";
				this.cycle = 10;
			}
		}
	}
	
	/**
	 * Determina qual o passo inicial da animao da ao.
	 * Depende de qual era a ao anterior para decidir isso.
	 */
	public void setStartStepFor(Action action) {
		if (action.getName().equals("Run") || action.getName().equals("Move") || action.getName().equals("Jump")){
			this.state = "Running";		
		}
		else {
			this.state = "Starting";
			this.speed = 0.01125;
			this.cycle = 10;
		}
	}
	
	/**
	 * Mtodo da super classe sobrescrito para retornar adequadamente o quadro da animao.
	 */
	public int getStep(){
		if (this.state.equals("Running")){
			return this.step + 10;
		}
		else if (this.state.equals("Starting")){
			return this.step;
		}
		else if (this.state.equals("Ending")){
			return this.step + 50;
		}
		return 0;
	}
}
