package model;

import model.Actions.Action;
import model.Actions.DoNothing;



public class Player {
	private String name;
	private boolean isMoving;
	private double dirX, dirY;
	private double x, y;
	private Action currentAction;
	private Action nextAction;
	private boolean hasCloned = false;
	
	/* Getters */
	
	public String getName(){
		return name;
	}

	public double getDirX() {
		return dirX;
	}
	
	public double getDirY() {
		return dirY;
	}
	
	public double getX() {
		return x;
	}
	
	public double getY() {
		return y;
	}
	
	public boolean isMoving() {
		return isMoving;
	}
	
	/**
	 * Mtodo para saber o nome da ao sendo executada.
	 * @return O nome da ao sendo executada.
	 */
	public String getCurrentAction(){
		if (this.currentAction == null){
			return "DoNothing";
		}
		return this.currentAction.getName();
	}
	
	/**
	 * Mtodo para verificar se o personagem j se clonou.
	 * @return true se ele  se clonou, false caso contrrio.
	 */
	public boolean isHasCloned() {
		return hasCloned;
	}
	
	/* Setters */
	
	/**
	 * Adiciona no next action do personagem a prxima ao a ser executada.
	 * Depois ele determina qual o ponto da animao que a ao ir comear dependendo da ao sendo executada.
	 */
	public void setNextAction(Action nextAction) {
		this.nextAction = nextAction;
		if (this.currentAction == null){
			this.currentAction = new DoNothing("DoNothing", this, null);
		}
		this.nextAction.setStartStepFor(this.currentAction);
	}
	
	public void setLocation(double x, double y) {
		this.x = x;
		this.y = y;
	}
	
	public void setDirection(double x, double y) {
		this.dirX = x;
		this.dirY = y;
	}
	
	public void setMoving(boolean isMoving) {
		this.isMoving = isMoving;
	}

	public void setHasCloned(boolean hasCloned) {
		this.hasCloned = hasCloned;
	}
	
	/**
	 * Construtor da classe Player.
	 * Inicializa o personagem com algums valores padro.
	 */
	public Player(){
		this.name = "Bob";
		this.x = 25;
		this.y = 15;
		this.isMoving = false;
		this.dirX = 1;
		this.dirY = 0;
		this.currentAction = null;
		this.nextAction = null;
	}
	
	/**
	 * Mtodo que verifica se o jogodor est executando uma ao e se existe uma prxima
	 * ao para executar. No final ele executa a ao corrente.
	 * Se ele estiver executando uma ao, ele avisa ela caso aparea uma prxima ao.
	 */
	public void performAction(){
		if (this.currentAction == null && this.nextAction == null){
			this.currentAction = new DoNothing("DoNothing", this, null);
			this.currentAction.initialize();
		}
		else if (this.currentAction == null){
			this.currentAction = this.nextAction;
			this.currentAction.initialize();
			this.nextAction = null;
		}
		if (this.nextAction != null){
			this.currentAction.setFlag(this.nextAction.getName());
		}
		this.currentAction.step();
	}
	
	/**
	 * Mtodo que verifica qual o passo de animao da ao.
	 * @return Um inteiro representando o passo da ao corrente.
	 */
	public int getStep(){
		if (this.currentAction == null){
			return 0;
		}
		return this.currentAction.getStep();
	}
	
	/**
	 * Finaliza a ao corrente.
	 */
	public void endAction(){
		Arena.instanceOf().setCliked(false);
		this.currentAction = null;
	}
}
