package view;

public class Camera {
	private double posx, posy, posz;  //Camera center position
	private double eyex, eyey, eyez;  //Camera focal point
	private int zoom;
	
	/**
	 * Construtor da classe Camera.
	 * Inicializa as variveis de posio e foco da cmera, bem como o zoom inicial dela
	 * @param posX  a posio x da cmera
	 * @param posY  a posio y da cmera
	 * @param posZ  a posio z da cmera
	 * @param eyeX  a posio x do foco da camera
	 * @param eyeY  a posio y do foco da camera
	 * @param eyeZ  a posio z do foco da camera
	 * @param Zoom  o zoom inicial
	 */
	public Camera(double posX, double posY, double posZ, double eyeX, double eyeY, double eyeZ, int Zoom){
		posx = posX;
		posy = posY;
		posz = posZ;
		eyex = eyeX;
		eyey = eyeY;
		eyez = eyeZ;
		zoom = Zoom;
	}
	
	/* Getters */	
	public double getXpos(){
		return posx;
	}
	
	public double getYpos(){
		return posy;
	}
	
	public double getZpos(){
		return posz;
	}

	public double getXeye(){
		return eyex;
	}
	
	public double getYeye(){
		return eyey;
	}
	
	public double getZeye(){
		return eyez;
	}
	
	public double getZoom(){
		return zoom;
	}
	
	/**
	 * MoveSideways move a posio da cmera para os lados,
	 * esse mtodo pode ser chamado pelo mouseInputHandler para 
	 * controlar a cmera atravs do mouse, no momento isso no  feito.
	 * (a cmera no momento segue o personagem selecionado, prefirimos dessa forma.)
	 * @param dist  a quantidade que se deseja mover a cmera
	 */
	public void moveSideways(float dist){
		double dirX, dirY, cameraEyeDist;
		
		cameraEyeDist = Math.sqrt((eyex - posx)*(eyex - posx) + (eyey - posy)*(eyey - posy));
		dirY = -(eyex - posx)/cameraEyeDist;
		dirX = (eyey - posy)/cameraEyeDist;
		posx -= dirX*dist/10; //A diviso por dez foi um ajuste fino para mover a cmera mais devagar; pode ser trocada
		posy -= dirY*dist/10;
	}
	
	/**
	 * MoveFoward move a posio da cmera para frente e para traz,
	 * esse mtodo pode ser chamado pelo mouseInputHandler para 
	 * controlar a cmera atravs do mouse, no momento isso no  feito.
	 * (a cmera no momento segue o personagem selecionado, prefirimos dessa forma.)
	 * @param dist  a quantidade que se deseja mover a cmera
	 */
	public void moveFoward(float dist){
		double dirX, dirY, cameraEyeDist;

		cameraEyeDist = Math.sqrt((eyex - posx)*(eyex - posx) + (eyey - posy)*(eyey - posy));
		dirX = (eyex - posx)/cameraEyeDist;
		dirY = (eyey - posy)/cameraEyeDist;

		posx -= dirX*dist/10;
		posy -= dirY*dist/10;
	}
	
	/**
	 * rotateZ rotaciona a cmera em torno do eixo Z, mantendo o foco igual e alterando a 
	 * posio da cmera. Quando chamado pelo mouseInputHandler ele permite um controle 
	 * maior da cmera atravs do mouse.
	 * @param angle  a quantidade que se deseja rotacionar a cmera, esse valor  submetido
	 * a um ajuste para melhor controle da cmera
	 */
	public void rotateZ(float angle){
		double currentAngle, newAngle, dist;
		
		dist = Math.sqrt((eyex - posx)*(eyex - posx) + (eyey - posy)*(eyey - posy));
		
		currentAngle = Math.toDegrees(Math.asin((eyey - posy)/dist));
		if(eyex > posx)
			currentAngle = 180 - currentAngle;
		
		newAngle = Math.toRadians((currentAngle + angle/3 + 360)%360);
		
		posy = eyey - dist*Math.sin(newAngle);		
		posx = eyex + dist*Math.cos(newAngle);
	}
	
	/**
	 * lowerCamera abaixa a posicao da cmera, mantendo o foco igual e alterando a 
	 * posio da cmera. Quando chamado pelo mouseInputHandler ele permite um controle 
	 * maior da cmera atravs do mouse.
	 * @param dist  a quantidade que se deseja abaixar a cmera, esse valor  submetido
	 * a um ajuste para melhor controle da cmera
	 */
	public void lowerCamera(float dist){
		posz -= dist/25;
		if(posz < 2)
			posz = 2;
		if(posz > 20)
			posz = 20;
	}
	
	/**
	 * changeZoom altera o zoom da cmera,  chamado pelo mouseInputHandler quando o wheel  movido.
	 * @param angle  a quantidade que se deseja alterar o zoom, esse valor  submetido
	 * a um ajuste para melhor controle da cmera
	 */
	public void changeZoom(int angle){
		zoom += 2*angle;
		if(zoom < 5)
			zoom = 5;
		if(zoom > 60)
			zoom = 60;
	}
	
	/**
	 * lookTo altera o foco da cmera, ao faz-lo tambm muda a posio dela de acordo.
	 * Com isso a cmera segue o seu foco.
	 * @param X  a coordenada x do novo foco da cmera
	 * @param Y  a coordenada y do novo foco da cmera
	 * @param Z  a coordenada z do novo foco da cmera
	 */
	public void lookTo(double X, double Y, double Z){
		double mx, my, mz;
		mx = eyex - X;
		my = eyey - Y;
		mz = eyez - Z;
		eyex -= mx;
		eyey -= my;
		eyez -= mz;
		posx -= mx;
		posy -= my;
		posz -= mz;
	}	
}
