package view;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.LinkedList;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;

import model.Arena;

import com.sun.opengl.util.FPSAnimator;

public class GLComponent3Dview extends GLCanvas implements GLEventListener {

	private static final long serialVersionUID = 9208387627341708344L;
	
	private static int larguraArena = 30;
	private static int comprimentoArena = 50;
	
	private int windowH;     //Altura da janela onde estamos desenhando a cena
	private int windowW;     //Largura da janela

	/* Matriz com indices das aes na lista interna do jogl, ser populada no init*/
	private int actions[][];
	/* Indices dos outros elementos da cena na lista interna do jogl*/
	private int arena, borda, chao, parede, ceu, selecionado;
	
	private GLU glu;
	private FPSAnimator animator;
	
	/* Instruo a ser impressa na tela, no canto da arena*/
	//private String instruct = "Move Bob!";
	private Camera camera;  //instancia da cmera
	
	/* A lista abaixo guarda as coordenadas de textura caso elas sejam definidas
	 * em algum .obj, dessa forma se elas no mudam (como acontece em geral
	 * em animaes) as coordenadas de texturas s precisariam ser definidas
	 * em um dos modelos (o primeiro a ser carregado)*/
	static private LinkedList<double []> text = new LinkedList<double[]>();
	static private boolean textureCoordsDefined = false;
			
	/**
	 * Construtor da classe GLComponent3Dview.
	 * Inicializa o GLcanvas da aplicao, a cmera, o vetor de indices de aes,
	 * o animador e componente GLU.
	 * @param fps  o nmero de frames por segundo que deseja-se manter na aplicao
	 */
	public GLComponent3Dview(int fps) {
		super(getCapabilities());
		addGLEventListener(this);
		glu = new GLU();
		animator = new FPSAnimator(this, fps);
		camera = new Camera(25, 5, 5, 25, 15, 1.5, 35);
		actions = new int[9][70];
	}

	/**
	 * getCapabilities inicializa um novo GLCapabilities e manda ele ser doubleBuffered
	 * e HardwareAccelerated, ao usar esse caps na inicializao do componente fazemos
	 * com que ele seja double buffered e tambm acelerado via hardware se possvel.
	 */
	private static GLCapabilities getCapabilities() {
		GLCapabilities caps = new GLCapabilities();
		caps.setDoubleBuffered(true);
		caps.setHardwareAccelerated(true);
		return caps;
	}

	/**
	 * init inicializa os componentes necessrios para o desenho da cena:
	 * Abilita luz e depthTest; pede para que as texturas sejam lidas;
	 * l todos os quadros de todas as animaes de aes de .objs e 
	 * deixa eles em uma glList para renderizao rpida depois;
	 * desenha os outros objetos da cena e coloca eles em glLists tambm;
	 * registra as classes de tratamento de mouse e teclado para serem chamadas
	 * nesses eventos; inicializa o animador.
	 * @param drawable  o GLAutoDrawable passado para o init pelo proprio JOGL, 
	 * que  quem chama esse mtodo, ele no  chamado pela nossa aplicao.
	 */
	public void init(GLAutoDrawable drawable) {
		GL gl = drawable.getGL();		
		gl.setSwapInterval(1);

		//Cores usadas em alguns materiais abaixo.
		float white[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		float black[] = { 0.0f, 0.0f, 0.0f, 1.0f };

		//Liga a luz e o depthTest
		gl.glEnable(GL.GL_CULL_FACE);
		gl.glEnable(GL.GL_LIGHTING);
		gl.glEnable(GL.GL_LIGHT0);
		gl.glEnable(GL.GL_DEPTH_TEST);
		
		//Pede para que o TextureHandler carregue as texturas dos arquivos de imagens
		TextureHandler.instanceOf().loadTextures();
		
		//Carrega do arquivo obj o primeiro objeto, no caso o Bob parado.
		//Nesse arquivo as coordenadas de textura so definidas. 
		actions[1][0] = gl.glGenLists(1);
		gl.glNewList(actions[1][0], GL.GL_COMPILE);
		player(gl, "database/meshes/player.obj");
		gl.glEndList();

		//Carrega o cliclo de animao do andar do Bob, nesses objs no precisamos
		//definir as texturas, as texturas definidas no player.obj foram usadas.
		for(int i=1; i<41; i++){
			String name = "walk"+i+".obj";			
			actions[0][i-1] = gl.glGenLists(1);
			gl.glNewList(actions[0][i-1], GL.GL_COMPILE);
			player(gl, "database/meshes/walk/"+name);
			gl.glEndList();
		}
		
		//Carrega o ciclo do jump para uma glList
		for(int i=1; i<41; i++){
			String name = "jump"+i+".obj";			
			actions[2][i-1] = gl.glGenLists(1);
			gl.glNewList(actions[2][i-1], GL.GL_COMPILE);
			player(gl, "database/meshes/jump/"+name);
			gl.glEndList();
		}
		
		//A segur carrega-se o restante das aes possveis e guarda-se o indice delas
		//na matriz de actions para quando desejarmos desenha-las no mtodo display
		for(int i=1; i<61; i++){
			String name = "run"+i+".obj";			
			actions[3][i-1] = gl.glGenLists(1);
			gl.glNewList(actions[3][i-1], GL.GL_COMPILE);
			player(gl, "database/meshes/run/"+name);
			gl.glEndList();
		}
		
		for(int i=1; i<41; i++){
			String name = "punch"+i+".obj";			
			actions[4][i-1] = gl.glGenLists(1);
			gl.glNewList(actions[4][i-1], GL.GL_COMPILE);
			player(gl, "database/meshes/punch/"+name);
			gl.glEndList();
		}
		
		for(int i=1; i<41; i++){
			String name = "wave"+i+".obj";			
			actions[5][i-1] = gl.glGenLists(1);
			gl.glNewList(actions[5][i-1], GL.GL_COMPILE);
			player(gl, "database/meshes/wave/"+name);
			gl.glEndList();
		}
		
		for(int i=1; i<41; i++){
			String name = "kick"+i+".obj";			
			actions[6][i-1] = gl.glGenLists(1);
			gl.glNewList(actions[6][i-1], GL.GL_COMPILE);
			player(gl, "database/meshes/kick/"+name);
			gl.glEndList();
		}
		
		for(int i=1; i<41; i++){
			String name = "hadoken"+i+".obj";			
			actions[7][i-1] = gl.glGenLists(1);
			gl.glNewList(actions[7][i-1], GL.GL_COMPILE);
			player(gl, "database/meshes/hadoken/"+name);
			gl.glEndList();
		}
		
		for(int i=1; i<41; i++){
			String name = "runningJump"+i+".obj";			
			actions[8][i-1] = gl.glGenLists(1);
			gl.glNewList(actions[8][i-1], GL.GL_COMPILE);
			player(gl, "database/meshes/runningJump/"+name);
			gl.glEndList();
		}
		
		//As coordenadas do Bob no so mais usadas agora, entao limpa-se o text
		text.clear();
		
        //O restante dos elementos da cena so desenhados direto pela aplicao, 
		//com exceo da parede, que ser tambm carregada de um .obj
		arena = gl.glGenLists(1);
		gl.glNewList(arena, GL.GL_COMPILE);
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, white, 0);
		arena(gl);
		gl.glEndList();
		
		borda = gl.glGenLists(1);
		gl.glNewList(borda, GL.GL_COMPILE);
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, white, 0);
		bordaArena(gl);
		gl.glEndList();
		
		chao = gl.glGenLists(1);
		gl.glNewList(chao, GL.GL_COMPILE);
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, white, 0);
		arena(gl);
		gl.glEndList();

		parede = gl.glGenLists(1);
		gl.glNewList(parede, GL.GL_COMPILE);
		player(gl, "database/meshes/wall.obj");
		gl.glEndList();
		
		ceu = gl.glGenLists(1);
		gl.glNewList(ceu, GL.GL_COMPILE);
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, white, 0);
		sky(gl);
		gl.glEndList();
		
		//Selecionado ser a pequena marcao que aparece no chao onde o usurio clica
		selecionado = gl.glGenLists(1);
		gl.glNewList(selecionado, GL.GL_COMPILE);
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, black, 0);
		ring(gl);
		gl.glEndList();

		gl.glEnable(GL.GL_NORMALIZE);

		//Registramos as classes para tratar eventos de mouse e teclado
		drawable.addMouseListener(MouseInputHandler3D.instanceOf());
		drawable.addMouseMotionListener(MouseInputHandler3D.instanceOf());
		drawable.addMouseWheelListener(MouseInputHandler3D.instanceOf());
		drawable.addKeyListener(KeyboardInputHandler.instanceOf());
		
		//Passamos a instncia de cmera sendo usada para que o mouse possa
		//control-la diretamente.
		MouseInputHandler3D.instanceOf().setCamera(camera);
		
		animator.start();
	}

	/**
	 * reshape trata eventos de quando a janela muda de tamanho, 
	 *  tambm chamado no inicio da aplicao pelo JOGL,
	 * para tratar o tamanho inicial da janela.
	 * @param drawable  o GLAutoDrawable passado pelo JOGL
	 * @param x  a posio horizontal do canto da janela
	 * @param y  a posio vertical do canto da janela 
	 * @param width  a largura da janela
	 * @param height  a altura da janela
	 */
	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
		GL gl = drawable.getGL();
		
		//Salva a altura e largura atual da tela
		windowH = height;
		windowW = width;
		//Avisa o mouseInputHandler do tamanho, para que ele possa tratar cliques.
		MouseInputHandler3D.instanceOf().setWindowSize(width,height);

		//Janela de visualizacao maior possivel
		gl.glViewport(0, 0, width, height);

		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();
		glu.gluPerspective(camera.getZoom(), (double)width/height, 2.0, 200.0);
		glu.gluLookAt(camera.getXpos(), camera.getYpos(), camera.getZpos(),
				camera.getXeye(), camera.getYeye(), camera.getZeye(), 0, 0, 1);
		gl.glMatrixMode(GL.GL_MODELVIEW);
	}

	/**
	 * display desenha a cena, ele  chamado pelo JOGL sempre que necessrio,
	 * ele limpa os buffers e redesenha a cena, atravs de chamadas para a 
	 * glList criada no init.
	 * @param drawable  o GLAutoDrawable passado pelo JOGL
	 */
	public void display(GLAutoDrawable drawable) {
		GL gl = drawable.getGL();
	
		//limpa os buffers necessrios
		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
		
		float white[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		
		//coloca a cmera no lugar certo e arruma o tamanho do volume de visualizao
		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();
		glu.gluPerspective(camera.getZoom(), (double)windowW/windowH, 2.0, 200.0);
		glu.gluLookAt(camera.getXpos(), camera.getYpos(), camera.getZpos(), camera.getXeye(), camera.getYeye(), camera.getZeye(), 0, 0, 1);
		gl.glMatrixMode(GL.GL_MODELVIEW);

		gl.glPushMatrix();

		//Posiciona a luz
		float pos[] = { -1, -1, 15, 0 };
		gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, pos, 0);
		
		//desenha a arena:
		/* Arena propria */
		gl.glEnable(GL.GL_TEXTURE_2D);
		TextureHandler.instanceOf().getTextureAt(0).bind();
		gl.glPushMatrix();
		gl.glCallList(arena);
		gl.glPopMatrix();
		gl.glDisable(GL.GL_TEXTURE_2D);
		
		/* borda da arena*/
		gl.glEnable(GL.GL_TEXTURE_2D);
		TextureHandler.instanceOf().getTextureAt(2).bind();
		gl.glPushMatrix();
		gl.glCallList(borda);
		gl.glPopMatrix();
		gl.glDisable(GL.GL_TEXTURE_2D);

		/* chao do lugar*/
		gl.glEnable(GL.GL_TEXTURE_2D);
		TextureHandler.instanceOf().getTextureAt(3).bind();
		gl.glPushMatrix();
		gl.glTranslated(-comprimentoArena, -larguraArena, 0);
		gl.glScaled(2.5, 2.5, 1);
		gl.glTranslated(5, 3, -0.9);
		gl.glCallList(chao);
		gl.glPopMatrix();
		gl.glDisable(GL.GL_TEXTURE_2D);
		
		/* parede em volta*/
		gl.glEnable(GL.GL_TEXTURE_2D);
		TextureHandler.instanceOf().getTextureAt(4).bind();
		gl.glPushMatrix();
		gl.glCallList(parede);
		gl.glPopMatrix();
		gl.glDisable(GL.GL_TEXTURE_2D);
		
		/* ceu acima*/
		gl.glEnable(GL.GL_TEXTURE_2D);
		TextureHandler.instanceOf().getTextureAt(5).bind();
		gl.glPushMatrix();
		gl.glCallList(ceu);
		gl.glPopMatrix();
		gl.glDisable(GL.GL_TEXTURE_2D);
		
		//Desenha o(s) Bob(s)
		double fatorX, fatorY, angle, dirX, dirY, dist;
		
		//Pega a posicao do Bob e poe o foco da camera nele
		fatorX = Arena.instanceOf().getSelected().getX();
		fatorY = Arena.instanceOf().getSelected().getY();
		camera.lookTo(fatorX, fatorY, 1.3);
		for (int i = 0; i < Arena.instanceOf().getBobs().size(); i++){
			//Pega a posicao do Bob que esta sendo desenhado
			fatorX = Arena.instanceOf().getBobs().get(i).getX();
			fatorY = Arena.instanceOf().getBobs().get(i).getY();
			//Pega direcao do Bob, transforma o vetor em unitario e calcula a rotacao do Bob
			dirX = Arena.instanceOf().getBobs().get(i).getDirX();
			dirY = Arena.instanceOf().getBobs().get(i).getDirY();
			dist = Math.sqrt(dirX*dirX + dirY*dirY);
			dirX = dirX/dist;
			dirY = dirY/dist;
			angle = Math.acos(dirX);
			if(dirY < 0)
				angle = -angle;
			//Bob  desenhado olhando para y negativo, nao x positivo, por isso o fator de 90
			angle = Math.toDegrees(angle) + 90; 
			
			//Aplica uma textura ao Bob, translada e rotacionao a matriz de desenho de acordo com a posicao e rotacao desse Bob
			gl.glEnable(GL.GL_TEXTURE_2D);
			TextureHandler.instanceOf().getTextureAt(1).bind();
			gl.glPushMatrix();
			gl.glTranslated(fatorX, fatorY, 0f);
			gl.glRotated(angle, 0, 0, 1);
			gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, white, 0);
			//chama a glList da ao que esse Bob est executando, no quadro em que ele est no momento.
			gl.glCallList(actions[getIndex(Arena.instanceOf().getBobs().get(i).getCurrentAction())][Arena.instanceOf().getBobs().get(i).getStep()]);
			gl.glPopMatrix();
			gl.glDisable(GL.GL_TEXTURE_2D);
		}
		
		//Codigo abaixo  usado para escrever algo na tela em 2D, no momento no  usado.
		/*
		GLUT glut = new GLUT();
		gl.glRasterPos3d(0, 0, 0);
		if(instruct != null)
			glut.glutBitmapString(GLUT.BITMAP_HELVETICA_12, instruct);	
		*/
		
		//Desenha a aura do ultimo click selecionado
		if(Arena.instanceOf().isCliked()){
			fatorX = Arena.instanceOf().getXClick();
			fatorY = Arena.instanceOf().getYClick();
			gl.glPushMatrix();
			gl.glTranslated(fatorX, fatorY, 0.01f);
			gl.glCallList(selecionado);
			gl.glPopMatrix();
		}
		
		gl.glPopMatrix();
	}

	public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {}

	/**
	 * player  responsvel por carregar um arquivo .obj e desenha-lo,
	 * ele  chamado para popular uma glList com os objetos carregados atravs de .objs
	 * @param gl  o GL do contexto onde a glList esta sendo criada, usado para desenhar o objeto
	 * @param path  o caminho e o nome do arquivo .obj de onde ele deve ler o objeto
	 */
	public static void player(GL gl, String path)
	{			
		GLU tool = new GLU();
		tool.gluQuadricTexture(tool.gluNewQuadric(), true);
		
		try {
			//Ferramentas para ler o arquivo de entrada
			FileInputStream stream = new FileInputStream(path);
			BufferedReader in = new BufferedReader(new InputStreamReader(stream));
			
			String s, splited[];
			LinkedList<double[]> vertex = new LinkedList<double[]>();
			LinkedList<double[]> normal = new LinkedList<double[]>();
			boolean normalsDefined = false;
			boolean texCoordDefinedHere = false;
			
			while(true){
				s = in.readLine();
				if(s == null)
					break;
				splited = s.split(" ");
				
				//Definio de vrtice na linha
				if(splited[0].compareTo("v")==0){
					if(splited.length != 4){
						System.out.println("Obj file with unexpected format");
						break;
					}
					double point[] = new double[3];
					point[0] = Double.parseDouble(splited[1]);  //x coordinate
					point[1] = Double.parseDouble(splited[2]);  //y coordinate
					point[2] = Double.parseDouble(splited[3]);  //z coordinate
					vertex.addLast(point);
				}
				//Definicao de normal de vrtice na linha
				else if(splited[0].compareTo("vn")==0){
					if(splited.length != 4){
						System.out.println("Obj file with unexpected format");
						break;
					}
					double point[] = new double[3];
					point[0] = Double.parseDouble(splited[1]);  //x coordinate
					point[1] = Double.parseDouble(splited[2]);  //y coordinate
					point[2] = Double.parseDouble(splited[3]);  //z coordinate
					normal.addLast(point);
					normalsDefined = true;
				}
				//Definicao de coordenada de textura na linha
				else if(splited[0].compareTo("vt")==0){
					if(splited.length != 4){
						System.out.println("Obj file with unexpected format");
						break;
					}
					double point[] = new double[3];
					point[0] = Double.parseDouble(splited[1]);  //x coordinate
					point[1] = Double.parseDouble(splited[2]);  //y coordinate
					point[2] = Double.parseDouble(splited[3]);  //z coordinate
					text.addLast(point);
					textureCoordsDefined = true;
					//Se as coordenadas foram definidas nesse arquivo precisarei fazer o parsing delas, senao usarei as ja guardadas.
					texCoordDefinedHere = true;
				}
				//Definio de face na linha
				else if(splited[0].compareTo("f")==0){
					if(splited.length < 4){
						System.out.println("Obj file with unexpected format");
						break;
					}
					/* Se as texturas foram definidas em outro arquivo, guarda-se o indice para pegar a 
					 * coordenada do vetor ja definido.*/
					int texCoordIndex = 1;
					
					gl.glBegin(GL.GL_POLYGON);
					if(normalsDefined && !textureCoordsDefined){
						for(int i=1; i<splited.length; i++){
							String split2[];
							split2 = splited[i].split("//");
							double [] v = vertex.get(Integer.parseInt(split2[0])-1);
							double [] vn = normal.get(Integer.parseInt(split2[1])-1);
							gl.glVertex3d(v[0], v[1], v[2]);
							gl.glNormal3d(vn[0], vn[1], vn[2]);
						}						
					}
					/* Se as coordenadas de textura e normais foram definidas nesse arquivo*/
					else if(normalsDefined && textureCoordsDefined && texCoordDefinedHere){
						for(int i=1; i<splited.length; i++){
							String split2[];
							split2 = splited[i].split("/");
							double [] v = vertex.get(Integer.parseInt(split2[0])-1);
							double [] vt = text.get(Integer.parseInt(split2[1])-1);
							double [] vn = normal.get(Integer.parseInt(split2[2])-1);
							gl.glTexCoord2d(vt[0], vt[1]);
							gl.glNormal3d(vn[0], vn[1], vn[2]);
							gl.glVertex3d(v[0], v[1], v[2]);
						}
					}
					/* Se as normais foram definidas aqui e as coordenadas de textura foram definidas em lugar.*/
					else if(normalsDefined && textureCoordsDefined && !texCoordDefinedHere){
						for(int i=1; i<splited.length; i++){
							String split2[];
							split2 = splited[i].split("//");
							double [] v = vertex.get(Integer.parseInt(split2[0])-1);
							double [] vt = text.get(texCoordIndex);
							texCoordIndex++;
							double [] vn = normal.get(Integer.parseInt(split2[1])-1);
							gl.glTexCoord2d(vt[0], vt[1]);
							gl.glVertex3d(v[0], v[1], v[2]);
							gl.glNormal3d(vn[0], vn[1], vn[2]);
						}
					}
					else if(!normalsDefined && !textureCoordsDefined){
						for(int i=1; i<splited.length; i++){
							double [] v = vertex.get(Integer.parseInt(splited[i])-1);
							gl.glVertex3d(v[0], v[1], v[2]);
						}
					}
					gl.glEnd();
				}
			}			
			in.close();
		} 
		catch (Exception e) {
			System.err.println("File input error");
			e.printStackTrace();
		}		
	}
	
	/**
	 * arena desenha a parte da arena onde o Bob anda, esse mtodo  chamado no init
	 * para criar a arena na glList
	 */
	public static void arena(GL gl)
	{
		/* desenha Arena (dividida em tiles de 2x2)*/
		for(int i=0; i<larguraArena; i=i+2){
			for(int j=0; j<comprimentoArena; j=j+2){
				gl.glBegin(GL.GL_QUADS);
				gl.glNormal3d(0, 0, 1);
	  			  gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(j, i+2, 0);
				  gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(j, i, 0);
				  gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(j+2, i, 0);
				  gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(j+2, i+2, 0);
				gl.glEnd();
			}
		}
	}
	
	/**
	 * bordaArena desenha a parte da arena onde o Bob anda, esse mtodo  chamado no init
	 * para criar a borda na glList, as coordenadas de textura so definidas tambm.
	 */
	public static void bordaArena(GL gl)
	{
		/* desenha a borda da arena (dividida em tiles de 1x2)*/
		/* Bordas horizontais*/
		for(int i=-1; i<=larguraArena; i=i+larguraArena+1){
			for(int j=0; j<comprimentoArena; j=j+2){
				gl.glBegin(GL.GL_QUADS);
				gl.glNormal3d(0, 0, 1);
	  			  gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(j, i+1, 0);
				  gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(j, i, 0);
				  gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(j+2, i, 0);
				  gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(j+2, i+1, 0);
				gl.glEnd();
			}
		}
		/* Bordas verticais*/
		for(int i=0; i<larguraArena; i=i+2){
			for(int j=-1; j<=comprimentoArena; j=j+comprimentoArena+1){
				gl.glBegin(GL.GL_QUADS);
				gl.glNormal3d(0, 0, 1);
	  			  gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(j, i+2, 0);
				  gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(j, i, 0);
				  gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(j+1, i, 0);
				  gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(j+1, i+2, 0);
				gl.glEnd();
			}
		}
		/* As 4 Quinas.*/
		gl.glBegin(GL.GL_QUADS);   /* Inferior esquerda*/
		gl.glNormal3d(0, 0, 1);
		gl.glTexCoord2f(1, 1); gl.glVertex3f(-1, 0, 0);
		gl.glTexCoord2f(1/2, 1); gl.glVertex3f(-1, -1, 0);
		gl.glTexCoord2f(1, 1); gl.glVertex3f(0, -1, 0);
		gl.glTexCoord2f(1, 0); gl.glVertex3f(0, 0, 0);
		gl.glEnd();
		gl.glBegin(GL.GL_QUADS);  /* Inferior direita*/
		gl.glNormal3d(0, 0, 1);
		gl.glTexCoord2f(1, 1); gl.glVertex3f(comprimentoArena, -1, 0);
		gl.glTexCoord2f(1/2, 1); gl.glVertex3f(comprimentoArena+1, -1, 0);
		gl.glTexCoord2f(1, 1); gl.glVertex3f(comprimentoArena+1, 0, 0);
		gl.glTexCoord2f(1, 0); gl.glVertex3f(comprimentoArena, 0, 0);
		gl.glEnd();
		gl.glBegin(GL.GL_QUADS); /* Superior esquerda*/
		gl.glNormal3d(0, 0, 1);
		gl.glTexCoord2f(1, 1); gl.glVertex3f(0, larguraArena+1, 0);
		gl.glTexCoord2f(1/2, 1); gl.glVertex3f(-1, larguraArena+1, 0);
		gl.glTexCoord2f(1, 1); gl.glVertex3f(-1, larguraArena, 0);
		gl.glTexCoord2f(1, 0); gl.glVertex3f(0, larguraArena, 0);
		gl.glEnd();
		gl.glBegin(GL.GL_QUADS); /* Superior direita*/
		gl.glNormal3d(0, 0, 1);
		gl.glTexCoord2f(1, 1); gl.glVertex3f(comprimentoArena, larguraArena+1, 0);
		gl.glTexCoord2f(1, 0); gl.glVertex3f(comprimentoArena, larguraArena, 0);
		gl.glTexCoord2f(1, 1); gl.glVertex3f(comprimentoArena+1, larguraArena, 0);
		gl.glTexCoord2f(1/2, 1); gl.glVertex3f(comprimentoArena+1, larguraArena+1, 0);
		gl.glEnd();
		
		/*Agora as bordas abaixo da arena, que a contornam */
		for(int j=-1; j<comprimentoArena; j=j+2){ /*Parte da frente*/
			gl.glBegin(GL.GL_QUADS);
			gl.glNormal3d(0, -1, 0);
			  gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(j, -1, 0);
			  gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(j, -1, -1);
			  gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(j+2, -1, -1);
			  gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(j+2, -1, 0);
			gl.glEnd();			
		}
		for(int j=-1; j<comprimentoArena; j=j+2){  /*Parte de traz*/
			gl.glBegin(GL.GL_QUADS); 
			gl.glNormal3d(0, 1, 0);
			  gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(j+2, larguraArena+1, 0);
			  gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(j+2, larguraArena+1, -1);
			  gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(j, larguraArena+1, -1);
			  gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(j, larguraArena+1, 0);
			gl.glEnd();			
		}
		for(int i=-1; i<larguraArena; i=i+2){ /*Parte da esquerda*/
			gl.glBegin(GL.GL_QUADS);
			gl.glNormal3d(-1, 0, 0);
			  gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(-1, i+2, 0);
			  gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(-1, i+2, -1);
			  gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(-1, i, -1);
			  gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(-1, i, 0);
			gl.glEnd();			
		}
		for(int i=-1; i<larguraArena; i=i+2){ /*Parte da direita*/
			gl.glBegin(GL.GL_QUADS);
			gl.glNormal3d(1, 0, 0);
			  gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(comprimentoArena+1, i, 0);
			  gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(comprimentoArena+1, i, -1);
			  gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(comprimentoArena+1, i+2, -1);
			  gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(comprimentoArena+1, i+2, 0);
			gl.glEnd();			
		}
	}
	
	/**
	 * sky desenha o cu da arena onde o Bob anda
	 */
	public static void sky(GL gl)
	{
		gl.glBegin(GL.GL_QUADS);
		  gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(-65, -50, 21);
		  gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(-65, 80,21);
		  gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(113, 80, 21);
		  gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(113, -50, 21);
		gl.glEnd();		
	}
	
	public static void ring(GL gl)
	{
		GLU tool = new GLU();
		tool.gluDisk(tool.gluNewQuadric(), 0.2, 0.4, 20, 2);		
	}
	
	/**
	 * setInstruct muda a instruo em texto que pode ser desenhada pelo JOGL.
	 * @param newInstruct  a string que passar a ser desenhada na tela caso a instrucao esteja sendo desenhada.
	 */
	public void setInstruct(String newInstruct){
		//instruct = newInstruct;
	}

	/**
	 * getIndex responde o indice de uma ao na matriz de indices da glList
	 * @param actionName  uma string com o nome da ao
	 * @return a linha daquela ao na matriz de indices
	 */
	public int getIndex(String actionName){
		if (actionName.equals("Move")){
			return 0;
		}
		else if (actionName.equals("Jump")){
			return 2;
		}
		else if (actionName.equals("Run")){
			return 3;
		}
		else if (actionName.equals("Teleport")){
			return 5;
		}
		else if (actionName.equals("Punch")){
			return 4;
		}
		else if (actionName.equals("Wave")){
			return 5;
		}
		else if (actionName.equals("Kick")){
			return 6;
		}
		else if (actionName.equals("Hadoken")){
			return 7;
		}
		else if (actionName.equals("RunningJump")){
			return 8;
		}
		else if (actionName.equals("Clone")){
			return 5;
		}
		else{
			return 1;
		}
	}
}


