package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.Border;

import controler.GameControler;
import controler.InputHandler;

public class Renderer extends JFrame implements ActionListener {
	private static final long serialVersionUID = 1428035987600346115L;
	
	/* instancia do componente JOGL, podem haver mais de uma dessas, inclusive para
	componentes descritos por classes diferentes que extendem GLCanvas, caso seja
	dejesado multiplos paineis com renderizaes JOGL. */
	private GLComponent3Dview glComponent3D;
	
	private static Renderer instance = null;

	//Botes da interface Swing
	private JButton jumpButton;
	private JButton punchButton;
	private JButton kickButton;
	private JButton waveButton;
	private JButton hadokenButton;
	private JButton cloneButton;
	private JButton fasterButton;
	private JButton slowerButton;
	private JButton closeButton;
	
	/**
	 * Construtor da classe Renderer.
	 * Armazena em instance a instncia da classe, para que exista apenas uma referncia.
	 * Inicializa a janela, inicializa os componentes e a torna visivel.
	 */
	public Renderer() {
		super("Moving Bob");

		Dimension d = getToolkit().getScreenSize();

		buildFrame(d);        
		initComponents();        
		setVisible(true);
		Renderer.instance = this;
	}
	
	/**
	 * Mtodo de acesso  instancia da classe Renderer.
	 * esse mtodo deve ser chamado depois da inicializacao da instncia atravs do construtor
	 * @return Uma instncia da classe Renderer.
	 */
	public static Renderer instanceOf(){
		return instance;
	}

	/**
	 * Mtodo que constri o frame principal da aplicao.
	 */
	private void buildFrame(Dimension d) {
		// Nicely center the window on the screen
		int width = 900;
		int x = (int)(d.getWidth()/2)-(int)(width/2);

		int height = 700;
		int y = (int)(d.getHeight()/2)-(int)(height/2);

		setBounds(x, y, width, height);
		setResizable(false);
		getContentPane().setLayout(new GridBagLayout());
		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
	}

	/**
	 * initComponents inicializa todos os componentes grficos da aplicao, 
	 * a interface Swing, os inputHandlers de mouse e teclado e os componentes JOGL.
	 * Alm disso ele coloca esses componentes em paineis na tela e dispe eles como 
	 * desejado, criando a interface da aplicao.
	 */
	private void initComponents() {
		Border border;
		GridBagConstraints constraints;

		// Engine components, os inputHandlers e o(s) componentes JOGL 
		new MouseInputHandler3D();
		new KeyboardInputHandler();
		new TextureHandler("database/Images/");
		glComponent3D = new GLComponent3Dview(60);
				
		// Botes de ao
		jumpButton = new JButton("Jump");
		jumpButton.addActionListener(this);
		punchButton = new JButton("Punch");
		punchButton.addActionListener(this);
		kickButton = new JButton("Kick");
		kickButton.addActionListener(this);
		waveButton = new JButton("Wave");
		waveButton.addActionListener(this);
		hadokenButton = new JButton("Hadoken");
		hadokenButton.addActionListener(this);
		cloneButton = new JButton("Clone");
		cloneButton.addActionListener(this);

		// Botes de opes
		fasterButton = new JButton("Faster");
		fasterButton.addActionListener(this);
		slowerButton = new JButton("Slower");
		slowerButton.addActionListener(this);
		closeButton = new JButton("Close");
		closeButton.addActionListener(this);
		
		// Painel com os botes de aes
		JPanel actionPanel = new JPanel(new GridBagLayout());
		border = BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(new Color(0.0f,0.0f,0.0f)),
				"Select Action");
		actionPanel.setBorder(border);        

		constraints = new GridBagConstraints();
		constraints.insets = new Insets(1,7,1,7);

		constraints.gridx = 0;
		constraints.gridy = 0;
		actionPanel.add(jumpButton, constraints);      
		constraints.gridx = 1;
		actionPanel.add(punchButton, constraints);
		constraints.gridx = 0;
		constraints.gridy = 1;
		actionPanel.add(kickButton, constraints);
		constraints.gridx = 1;
		actionPanel.add(waveButton, constraints);
		constraints.gridx = 0;
		constraints.gridy = 2;
		actionPanel.add(cloneButton, constraints);
		constraints.gridx = 1;
		actionPanel.add(hadokenButton, constraints);
		
		// Painel com os botes de opes
		JPanel optionsPanel = new JPanel(new GridBagLayout());
		constraints = new GridBagConstraints();
		border = BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(new Color(0.0f,0.0f,0.0f)),
				"Options");
		optionsPanel.setBorder(border);
		constraints.insets = new Insets(3,3,3,3);
		optionsPanel.add(fasterButton, constraints);
		optionsPanel.add(slowerButton, constraints);
		optionsPanel.add(closeButton, constraints);

		// O painel contendo o componente OpenGL
		JPanel glPanel = new JPanel(new BorderLayout());    
		glPanel.setPreferredSize(new Dimension(getBounds().width-10, (int)getBounds().height*3/4));
		glPanel.add(glComponent3D);        

		constraints = new GridBagConstraints();
		constraints.weightx = constraints.weighty = 1.0d;
		constraints.fill = GridBagConstraints.BOTH;

		constraints.gridx = 0;
		constraints.gridy = 0;
		glPanel.setBorder(BorderFactory.createRaisedBevelBorder());
		getContentPane().add(glPanel, constraints);

		// O painel de fundo que segura todos os outros
		JPanel bottomPanel = new JPanel(new GridLayout(1,3,3,3));
		bottomPanel.add(actionPanel);
		bottomPanel.add(optionsPanel);        
		constraints.gridx = 0;
		constraints.gridy = GridBagConstraints.RELATIVE;
		getContentPane().add(bottomPanel, constraints);
	}

	/**
	 * ActionPerformed  responsvel por responder quando um dos botes Swing 
	 * precionado, no caso ele apenas informa qual boto foi pressionado para o 
	 * inputHandler, que dever tratar o comando do usurio adequadamente.
	 */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource().equals(closeButton)) {
			setVisible(false);
			dispose();
			System.exit(0);
		}
		else if(e.getSource().equals(fasterButton)) {
			GameControler.instanceOf().faster();
		}
		else if(e.getSource().equals(slowerButton)) {
			GameControler.instanceOf().slower();
		}
		//Algum boto de ao jump/punch/Kick/wave/hadoken
		else{
			InputHandler.instanceOf().pressButton(e.getActionCommand());
		}
	}
	
	/**
	 * setInstruct informa o componente JOGL que houve uma mudana na instruo 
	 * que se deseja impressa na tela.
	 */
	public void setInstruct(String newInstruct){
		glComponent3D.setInstruct(newInstruct);
	}
}